/*
 * MINIGUI - Harbour Win32 GUI library
 *
 * Copyright 2014 Verchenko Andrey <verchenkoag@gmail.com>
 * Copyright 2014 Pavel Tsarenko <tpe2@mail.ru>
 *
 *          ?
 * How do you put a file in resources and get him out of there ?
 * 
*/

#include "minigui.ch"

#define PATH_FILE   GetStartupFolder() + "\"
////////////////////////////////////////////////////////////////////////////
FUNCTION WriteDiskMyFile() 
   LOCAL cFileDbf1 := PATH_FILE + "test1.dbf"
   LOCAL cFileDbf2 := PATH_FILE + "test2.dbf"
   LOCAL cFileDot  := PATH_FILE + "test2shablon.doc"

   // Unloading function of resource
   IF !FILE( cFileDbf1 )   
      IF !ResourceWriteDisk( cFileDbf1, "test1_dbf" )   
         MsgStop ("Could not create file on disk!!" + CRLF + CRLF + ;
                   cFileDbf1 + CRLF )
      ENDIF
   ENDIF
   IF !FILE( cFileDbf2 )   
      IF !ResourceWriteDisk( cFileDbf2, "test2_dbf" )   
         MsgStop ("Could not create file on disk!!" + CRLF + CRLF + ;
                   cFileDbf2 + CRLF )
      ENDIF
   ENDIF
   IF !FILE( cFileDot )   
      IF !ResourceWriteDisk( cFileDot, "test2shablon_doc" )   
         MsgStop ("Could not create file on disk!!" + CRLF + CRLF + ;
                   cFileDot + CRLF )
      ENDIF
   ENDIF

RETURN NIL

///////////////////////////////////////////////////////////////////////////////
//// Pavel Tsarenko ///////////////////////////////////////////////////////////
Function ResourceWriteDisk( cFile, cResName, cType )
   hb_default( @cType, "CUSTOM" )
Return hb_memowrit( cFile, my_loadresource( cResName, cType ) )

//// Pavel Tsarenko //////////////////////////////////////////////////////////
#pragma BEGINDUMP

#include "hbapi.h"
#include "hbwinuni.h"
#include "windows.h"


HB_FUNC( MY_LOADRESOURCE )
{
   HANDLE hInstance = NULL;

   /* Set default return value */
   hb_retc_null();

   if( !hb_winmainArgGet( &hInstance, NULL, NULL ) )
      hInstance = GetModuleHandle( NULL );


   if( hInstance )
   {
      void * hName;
      void * hType;

      HRSRC hRes = FindResource( ( HMODULE ) hInstance,
                                 HB_PARSTRDEF( 1, &hName, NULL ),
                                 HB_PARSTRDEF( 2, &hType, NULL ) );

      if( hRes )
      {
         HGLOBAL hMem = LoadResource( NULL, hRes );

         if( hMem )
         {
            void * pMem = LockResource( hMem );

            if( pMem )
               hb_retclen( ( char * ) pMem, SizeofResource( NULL, hRes ) );
         }
      }

      hb_strfree( hName );
      hb_strfree( hType );
   }
}

HB_FUNC( GETFILEVERSIONINFO )
{
   const char * szFile  = hb_parc( 1 );
   UINT uAction   = HB_ISNUM( 2 ) ? hb_parni( 2 ) : 1;
   DWORD dwHandle = 0;
   DWORD dwSize   = GetFileVersionInfoSize( szFile, &dwHandle );
   char * szOut   = NULL;
   BOOL  bOk      = FALSE;

   if( dwSize )
   {
      char * szBlock = ( char * ) hb_xgrab( 255 );
      char * szSubBlock = ( char * ) hb_xgrab( 255 );
      HGLOBAL hMem = GlobalAlloc( GMEM_MOVEABLE, dwSize );
      VS_FIXEDFILEINFO * vsInfo;
      UINT nLen = 0;

      if( hMem )
      {
         LPVOID pMem = GlobalLock( hMem );

         if( pMem && GetFileVersionInfo( szFile, dwHandle, dwSize, pMem ) )
         {
            if( VerQueryValue( pMem, "\\VarFileInfo\\Translation", ( LPVOID * ) &vsInfo, &nLen ) )
            {
               *( LPDWORD ) vsInfo = MAKELONG( HIWORD( *( LPDWORD ) vsInfo ), LOWORD( *( LPDWORD ) vsInfo ) );

               sprintf( szBlock, "\\StringFileInfo\\%08lx\\", *( LPDWORD )( vsInfo ) );

               switch( uAction ) {
                  case 1:
                     sprintf( szSubBlock, "%s%s", szBlock, "Comments" );
                     break;

                  case 2:
                     sprintf( szSubBlock, "%s%s", szBlock, "CompanyName" );
                     break;

                  case 3:
                     sprintf( szSubBlock, "%s%s", szBlock, "FileDescription" );
                     break;

                  case 4:
                     sprintf( szSubBlock, "%s%s", szBlock, "FileVersion" );
                     break;

                  case 5:
                     sprintf( szSubBlock, "%s%s", szBlock, "InternalName" );
                     break;

                  case 6:
                     sprintf( szSubBlock, "%s%s", szBlock, "LegalCopyright" );
                     break;

                  case 7:
                     sprintf( szSubBlock, "%s%s", szBlock, "LegalTrademarks" );
                     break;

                  case 8:
                     sprintf( szSubBlock, "%s%s", szBlock, "OriginalFilename" );
                     break;

                  case 9:
                     sprintf( szSubBlock, "%s%s", szBlock, "PrivateBuild" );
                     break;

                  case 10:
                     sprintf( szSubBlock, "%s%s", szBlock, "ProductName" );
                     break;

                  case 11:
                     sprintf( szSubBlock, "%s%s", szBlock, "ProductVersion" );
                     break;

                  case 12:
                     sprintf( szSubBlock, "%s%s", szBlock, "SpecialBuild" );
                     break;
               }

               if( VerQueryValue( pMem, szSubBlock, ( LPVOID * ) &szOut, &nLen ) )
                  bOk = TRUE;

               hb_xfree( szBlock );
               hb_xfree( szSubBlock );
            }

            GlobalUnlock( hMem );
            GlobalFree( hMem );
         }
      }
   }

   if( bOk )
      hb_retc( szOut );
   else
      hb_retc( "" );
}

#pragma ENDDUMP
